/*
   Copyright 2009 Timo Schneider <timos@hrz.tu-chemnitz.de>
   
   This file is part of Tgtd.
   
   Tgtd is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
   
   Tgtd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with Tgtd.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Storage.hpp"
#include "Bot.hpp"
#include "cmdline.h"

log4cxx::LoggerPtr Bot::logger(log4cxx::Logger::getLogger("Bot"));
log4cxx::LoggerPtr Storage::logger(log4cxx::Logger::getLogger("Storage"));
log4cxx::LoggerPtr logger(log4cxx::Logger::getLogger("Main"));


bool fileExists(const std::string& fileName) {

	std::fstream fin;
	
	fin.open(fileName.c_str(),std::ios::in);
	if (fin.is_open()) {
		fin.close();
		return true;
	}
	fin.close();
	return false;
}


void parse_options (int* argc, char*** argv, Bot* b) {
	
	gengetopt_args_info args_info;
	cmdline_parser_params parser_params;

	// do _not_ check for all required params when parsing the options,
	// we will manually check after we parsed cmdline as well as conffile
	// options.
	cmdline_parser_params_init(&parser_params);
	parser_params.check_required = 0;

	if (cmdline_parser_ext(*argc, *argv, &args_info, &parser_params) != 0) {
		LOG4CXX_FATAL(logger, "Unable to parse command line options.");
		std::exit(EXIT_FAILURE);
	}
	
	if (fileExists(args_info.configfile_arg)) {
		parser_params.initialize = 0;

		if (cmdline_parser_config_file(args_info.configfile_arg, &args_info, &parser_params) != 0) {
			LOG4CXX_FATAL(logger, "Unable to parse config file options.");
			std::exit(EXIT_FAILURE);
		}
	}
	
	// check for required options
	cmdline_parser_required(&args_info, (*argv)[0]);


	// now we finished the actual parsing, we just need to apply the options now

	log4cxx::BasicConfigurator::resetConfiguration();
	if (args_info.logfile_given) {
		log4cxx::FileAppender * fileAppender = new log4cxx::FileAppender(log4cxx::LayoutPtr(new log4cxx::SimpleLayout()), args_info.logfile_arg, true);
		log4cxx::helpers::Pool p;
		fileAppender->activateOptions(p);
		log4cxx::BasicConfigurator::configure(log4cxx::AppenderPtr(fileAppender));
		log4cxx::Logger::getRootLogger()->setLevel(log4cxx::Level::toLevel(args_info.verbositylevel_arg));
	}
	else {			
		log4cxx::BasicConfigurator::configure();
		log4cxx::Logger::getRootLogger()->setLevel(log4cxx::Level::toLevel(args_info.verbositylevel_arg));
	}

	std::string jid;
	jid.append(args_info.username_arg);
	jid.append("@");
	jid.append(args_info.jabberserver_arg);
	jid.append("/");
	jid.append(args_info.ressource_arg);

	Storage* s = new Storage(args_info.database_arg);
	
	b->set_storage(s);
	b->set_jid_full(jid);
	b->set_password(args_info.password_arg);

}

int main( int argc, char** argv ) {
	
	// We need this simple logger so that we can log error messages
	// that occur _before_ we actually know where and what we should log
	// from the commandline/configfile options
	log4cxx::BasicConfigurator::configure();
	
	Bot b;
	parse_options(&argc, &argv, &b);
	LOG4CXX_INFO(logger, "Starting tgtdbot.");
	b.start_bot();

}

